@@ -94,8 +94,8 @@ class Export(QDialog):
         self.close_button.setVisible(False)
         self.exporting = False
 
-        # Pause playback (to prevent crash since we are fixing to change the timeline's max size)
-        get_app().window.actionPlay_trigger(None, force="pause")
+        # Pause playback
+        get_app().window.PauseSignal.emit()
 
         # Hide audio channels
         self.lblChannels.setVisible(False)
@@ -138,17 +138,8 @@ class Export(QDialog):
         self.timeline.Open()
 
         # Default export path
-        recommended_path = os.path.join(info.HOME_PATH)
-        if get_app().project.current_filepath:
-            recommended_path = os.path.dirname(get_app().project.current_filepath)
-
-        export_path = get_app().project.get("export_path")
-        if export_path and os.path.exists(export_path):
-            # Use last selected export path
-            self.txtExportFolder.setText(export_path)
-        else:
-            # Default to home dir
-            self.txtExportFolder.setText(recommended_path)
+        recommended_path = self.s.getDefaultPath(self.s.actionType.EXPORT)
+        self.txtExportFolder.setText(recommended_path)
 
         # Is this a saved project?
         if not get_app().project.current_filepath:
@@ -219,14 +210,14 @@ class Export(QDialog):
         self.profile_names.sort()
 
         # Loop through sorted profiles
-        self.selected_profile_index = 0
+        self.selected_profile = None
         for box_index, profile_name in enumerate(self.profile_names):
             # Add to dropdown
             self.cboProfile.addItem(self.getProfileName(self.getProfilePath(profile_name)), self.getProfilePath(profile_name))
 
-            # Set default (if it matches the project)
+            # Set default profile (if it matches the project)
             if get_app().project.get(['profile']) in profile_name:
-                self.selected_profile_index = box_index
+                self.selected_profile = profile_name
 
         # ********* Simple Project Type **********
         # load the simple project type dropdown
@@ -244,17 +235,18 @@ class Export(QDialog):
                     # This indicates an invalid Preset file - display an error and continue
                     log.error("Failed to parse file '%s' as a preset: %s" % (preset_path, e))
 
-        # Exclude duplicates
-        selected_type = 0
+        # Add first project type (always add this one first)
+        all_formats_text = _("All Formats")
+        self.cboSimpleProjectType.addItem(all_formats_text, all_formats_text)
+
+        # Add remaining project types (exclude duplicates)
         presets = list(set(presets))
-        for type_index, item in enumerate(sorted(presets)):
-            self.cboSimpleProjectType.addItem(item, item)
-            if item == _("All Formats"):
-                selected_type = type_index
+        for item in sorted(presets):
+            if item != all_formats_text:
+                self.cboSimpleProjectType.addItem(item, item)
 
         # Always select 'All Formats' option
-        self.cboSimpleProjectType.setCurrentIndex(selected_type)
-
+        self.cboSimpleProjectType.setCurrentIndex(0)
 
         # Populate all profiles
         self.populateAllProfiles(get_app().project.get(['profile']))
@@ -451,9 +443,9 @@ class Export(QDialog):
             previous_quality = self.cboSimpleQuality.currentIndex()
             if previous_quality < 0:
                 previous_quality = self.cboSimpleQuality.count() - 1
-            previous_profile = self.cboSimpleVideoProfile.currentIndex()
-            if previous_profile < 0:
-                previous_profile = self.selected_profile_index
+            previous_profile = self.cboSimpleVideoProfile.currentText()
+            if previous_profile:
+                self.selected_profile = previous_profile
             self.cboSimpleVideoProfile.clear()
             self.cboSimpleQuality.clear()
 
@@ -543,9 +535,15 @@ class Export(QDialog):
             for item in sorted(profiles_list):
                 self.cboSimpleVideoProfile.addItem(self.getProfileName(self.getProfilePath(item)), self.getProfilePath(item))
 
-            if all_profiles:
-                # select the project's current profile
-                self.cboSimpleVideoProfile.setCurrentIndex(previous_profile)
+            # select the project's current profile
+            profile_index = self.getVideoProfileIndex(self.selected_profile)
+            if profile_index != -1:
+                # Re-select project profile (if found in list)
+                self.cboSimpleVideoProfile.setCurrentIndex(profile_index)
+            else:
+                # Previous profile not in list, so
+                # default to first profile in list
+                self.cboSimpleVideoProfile.setCurrentIndex(0)
 
             # set the quality combo
             # only populate with quality settings that exist
@@ -562,6 +560,14 @@ class Export(QDialog):
             else:
                 self.cboSimpleQuality.setCurrentIndex(self.cboSimpleQuality.count() - 1)
 
+    def getVideoProfileIndex(self, profile_name):
+        """Get the index of a profile name (-1 if not found)"""
+        for index in range(self.cboSimpleVideoProfile.count()):
+            combo_profile = self.cboSimpleVideoProfile.itemText(index)
+            if combo_profile == profile_name:
+                return index
+        return -1
+
     def cboSimpleVideoProfile_index_changed(self, widget, index):
         selected_profile_path = widget.itemData(index)
         log.info(selected_profile_path)
@@ -596,11 +602,16 @@ class Export(QDialog):
 
         # get translations
         _ = get_app()._tr
+        default_path = self.s.getDefaultPath(self.s.actionType.EXPORT)
 
         # update export folder path
-        file_path = QFileDialog.getExistingDirectory(self, _("Choose a Folder..."), self.txtExportFolder.text())
+        file_path = QFileDialog.getExistingDirectory(self,
+                                                     _("Choose a Folder..."),
+                                                     default_path)
 
+        # Don't change path if chosen path isn't valid
         if os.path.exists(file_path):
+            self.s.setDefaultPath(self.s.actionType.EXPORT, file_path)
             self.txtExportFolder.setText(file_path)
 
     def convert_to_bytes(self, BitRateString):
@@ -726,7 +737,15 @@ class Export(QDialog):
             if not file_name_with_ext.endswith(file_ext):
                 file_name_with_ext = '{}.{}'.format(file_name_with_ext, file_ext)
 
-        export_file_path = os.path.join(self.txtExportFolder.text().strip() or default_folder, file_name_with_ext)
+        # Remove trailing whitespace, unless such a folder exists.
+        folder_path = self.txtExportFolder.text().lstrip()
+        if folder_path and not os.path.isdir(folder_path):
+            log.debug("Folder path does not exist. Removing trailing whitespace.")
+            if os.path.isdir(folder_path.rstrip()):
+                log.debug("Directory %s does exist. Using it instead." % folder_path)
+                folder_path = folder_path.rstrip()
+
+        export_file_path = os.path.join(folder_path or default_folder, file_name_with_ext)
         log.info("Export path: %s" % export_file_path)
 
         # Check if filename is valid (by creating a blank file in a temporary place)
@@ -794,7 +813,8 @@ class Export(QDialog):
                 video_settings["vcodec"] = image_ext
 
         # Store updated export folder path in project file
-        get_app().updates.update_untracked(["export_path"], os.path.dirname(export_file_path))
+        settings = get_app().get_settings()
+        settings.setDefaultPath(settings.actionType.EXPORT, export_file_path)
         # Mark project file as unsaved
         get_app().project.has_unsaved_changes = True
 
@@ -819,6 +839,12 @@ class Export(QDialog):
         # Apply mappers to timeline readers
         self.timeline.ApplyMapperToClips()
 
+        # Initialize
+        max_frame = 0
+
+        # Precision of the progress bar
+        format_of_progress_string = "%4.1f%% "
+
         # Create FFmpegWriter
         try:
             w = openshot.FFmpegWriter(export_file_path)
@@ -880,13 +906,8 @@ class Export(QDialog):
             end_frame_export = video_settings.get("end_frame")
             last_exported_time = time.time()
             last_displayed_exported_portion = 0.0
-            current_exported_portion = 0.0
-            digits_after_decimalpoint = 1
-            # Precision of the progress bar
-            format_of_progress_string = "%4.1f%% "
 
             # Write each frame in the selected range
-            max_frame = 0
             for frame in range(video_settings.get("start_frame"), video_settings.get("end_frame") + 1):
                 # Update progress bar (emit signal to main window)
                 end_time_export = time.time()
@@ -902,12 +923,12 @@ class Export(QDialog):
                         # We want at least 1 digit after the decimal point
                         digits_after_decimalpoint = 1
                     if digits_after_decimalpoint > 5:
-                        # We don't want not more than 5 difits after the decimal point
+                        # We don't want not more than 5 digits after the decimal point
                         digits_after_decimalpoint = 5
                     last_displayed_exported_portion = current_exported_portion
                     format_of_progress_string = "%4." + str(digits_after_decimalpoint) + "f%% "
                     last_exported_time = time.time()
-                    if ((( frame - start_frame_export ) != 0) & (( end_time_export - start_time_export ) != 0)):
+                    if ((frame - start_frame_export) != 0) & ((end_time_export - start_time_export) != 0):
                         seconds_left = round(( start_time_export - end_time_export )*( frame - end_frame_export )/( frame - start_frame_export ))
                         fps_encode = ((frame - start_frame_export)/(end_time_export-start_time_export))
                         if frame == end_frame_export:
@@ -924,12 +945,12 @@ class Export(QDialog):
                         format_of_progress_string
                     )
 
-                    # track largest frame processed
-                    max_frame = frame
-
                     # Process events (to show the progress bar moving)
                     QCoreApplication.processEvents()
 
+                # track largest frame processed
+                max_frame = frame
+
                 # Write the frame object to the video
                 w.WriteFrame(self.timeline.GetFrame(frame))
 
@@ -1008,17 +1029,6 @@ class Export(QDialog):
             # Reveal done button
             self.close_button.setVisible(True)
 
-            # Restore windows title to show elapsed time
-            title_message = titlestring(seconds_run, fps_encode, "Elapsed")
-
-            self.ExportFrame.emit(
-                title_message,
-                video_settings.get("start_frame"),
-                video_settings.get("end_frame"),
-                frame,
-                format_of_progress_string
-            )
-
             # Make progress bar green (to indicate we are done)
             from PyQt5.QtGui import QPalette
             p = QPalette()
