@@ -35,6 +35,7 @@ from PyQt5.QtGui import (
     QPixmap, QColor,
     )
 
+from classes.waveform import get_audio_data
 from classes import info, updates
 from classes import openshot_rc  # noqa
 from classes.query import Clip, Transition, Effect
@@ -80,8 +81,12 @@ class PropertiesModel(updates.UpdateInterface):
         self.next_item_id = item_id
         self.next_item_type = item_type
 
-        # Update the model data
-        self.update_timer.start()
+        if not item_type and self.selected:
+            # immediately clear property selection (but only if needed)
+            self.update_item_timeout()
+        else:
+            # Delay property selection (to prevent spamming of multi-selections)
+            self.update_timer.start()
 
     # Update the next item (once the timer runs out)
     def update_item_timeout(self):
@@ -95,31 +100,33 @@ class PropertiesModel(updates.UpdateInterface):
 
         log.debug("Update item: %s" % item_type)
 
-        timeline = get_app().window.timeline_sync.timeline
-        if item_type == "clip":
-            c = timeline.GetClip(item_id)
-            if c:
-                # Append to selected items
-                self.selected.append((c, item_type))
-        if item_type == "transition":
-            t = timeline.GetEffect(item_id)
-            if t:
-                # Append to selected items
-                self.selected.append((t, item_type))
-        if item_type == "effect":
-            e = timeline.GetClipEffect(item_id)
-            if e:
-                # Filter out basic properties, since this is an effect on a clip
-                self.filter_base_properties = ["position", "layer", "start", "end", "duration"]
-                # Append to selected items
-                self.selected.append((e, item_type))
-                self.selected_parent = e.ParentClip()
+        if item_type:
+            # Only retrieve object if item_type is valid
+            timeline = get_app().window.timeline_sync.timeline
+            if item_type == "clip":
+                c = timeline.GetClip(item_id)
+                if c:
+                    # Append to selected items
+                    self.selected.append((c, item_type))
+            if item_type == "transition":
+                t = timeline.GetEffect(item_id)
+                if t:
+                    # Append to selected items
+                    self.selected.append((t, item_type))
+            if item_type == "effect":
+                e = timeline.GetClipEffect(item_id)
+                if e:
+                    # Filter out basic properties, since this is an effect on a clip
+                    self.filter_base_properties = ["position", "layer", "start", "end", "duration"]
+                    # Append to selected items
+                    self.selected.append((e, item_type))
+                    self.selected_parent = e.ParentClip()
 
-        # Update frame # from timeline
-        self.update_frame(get_app().window.preview_thread.player.Position(), reload_model=False)
+            # Update frame # from timeline
+            self.update_frame(get_app().window.preview_thread.player.Position(), reload_model=False)
 
-        # Get ID of item
-        self.new_item = True
+            # Get ID of item
+            self.new_item = True
 
         # Update the model data
         self.update_model(get_app().window.txtPropertyFilter.text())
@@ -148,23 +155,26 @@ class PropertiesModel(updates.UpdateInterface):
 
             # Determine the frame needed for this clip (based on the position on the timeline)
             time_diff = (requested_time - clip.Position()) + clip.Start()
-            self.frame_number = round(time_diff * fps_float) + 1
+            new_frame_number = round(time_diff * fps_float) + 1
+            if new_frame_number != self.frame_number:
+                # If frame # has changed
+                self.frame_number = new_frame_number
 
-            # Calculate biggest and smallest possible frames
-            min_frame_number = round((clip.Start() * fps_float)) + 1
-            max_frame_number = round((clip.End() * fps_float)) + 1
+                # Calculate biggest and smallest possible frames
+                min_frame_number = round((clip.Start() * fps_float)) + 1
+                max_frame_number = round((clip.End() * fps_float)) + 1
 
-            # Adjust frame number if out of range
-            if self.frame_number < min_frame_number:
-                self.frame_number = min_frame_number
-            if self.frame_number > max_frame_number:
-                self.frame_number = max_frame_number
+                # Adjust frame number if out of range
+                if self.frame_number < min_frame_number:
+                    self.frame_number = min_frame_number
+                if self.frame_number > max_frame_number:
+                    self.frame_number = max_frame_number
 
-            log.debug("Update frame to %s" % self.frame_number)
+                log.debug("Update frame to %s" % self.frame_number)
 
-            # Update the model data
-            if reload_model:
-                self.update_model(get_app().window.txtPropertyFilter.text())
+                # Update the model data
+                if reload_model:
+                    self.update_model(get_app().window.txtPropertyFilter.text())
 
     def remove_keyframe(self, item):
         """Remove an existing keyframe (if any)"""
@@ -237,6 +247,14 @@ class PropertiesModel(updates.UpdateInterface):
                     log.debug("Found point to delete at X=%s" % point_to_delete["co"]["X"])
                     keyframe["Points"].remove(point_to_delete)
 
+            # Determine if waveforms are impacted by this change
+            has_waveform = False
+            waveform_file_id = None
+            if property_key == "volume":
+                if clip_data.get("ui", {}).get("audio_data", []):
+                    waveform_file_id = c.data.get("file_id")
+                    has_waveform = True
+
             # Reduce # of clip properties we are saving (performance boost)
             clip_data = {property_key: clip_data[property_key]}
             if object_id:
@@ -247,6 +265,10 @@ class PropertiesModel(updates.UpdateInterface):
                 # Save
                 c.save()
 
+                # Update waveforms (if needed)
+                if has_waveform:
+                    get_audio_data({waveform_file_id: [c.id]})
+
                 # Update the preview
                 get_app().window.refreshFrameSignal.emit()
 
@@ -315,33 +337,35 @@ class PropertiesModel(updates.UpdateInterface):
                                 break
 
                             elif interpolation > -1 and point["co"]["X"] == previous_point_x:
-                                # Only update interpolation type (and the LEFT side of the curve)
+                                # Only update the LEFT side of the curve (i.e. the previous point's right handle)
                                 found_point = True
                                 clip_updated = True
-                                point["interpolation"] = interpolation
-                                if interpolation == 0:
+                                if point.get("interpolation", 2) == 0 and interpolation_details:
                                     point["handle_right"] = point.get("handle_right") or {"Y": 0.0, "X": 0.0}
                                     point["handle_right"]["X"] = interpolation_details[0]
                                     point["handle_right"]["Y"] = interpolation_details[1]
 
-                                log.debug(
-                                    "updating interpolation mode point: co.X = %d to %d",
-                                    point["co"]["X"], interpolation)
-                                log.debug("use interpolation preset: %s", str(interpolation_details))
+                                    log.debug("updating previous point (right handle): co.X = %d", point["co"]["X"])
+                                    log.debug("use interpolation preset: %s", str(interpolation_details))
+                                else:
+                                    # Remove unused bezier property
+                                    point.pop("handle_right", None)
 
                             elif interpolation > -1 and point["co"]["X"] == closest_point_x:
                                 # Only update interpolation type (and the RIGHT side of the curve)
                                 found_point = True
                                 clip_updated = True
                                 point["interpolation"] = interpolation
-                                if interpolation == 0:
+                                if interpolation == 0 and interpolation_details:
                                     point["handle_left"] = point.get("handle_left") or {"Y": 0.0, "X": 0.0}
                                     point["handle_left"]["X"] = interpolation_details[2]
                                     point["handle_left"]["Y"] = interpolation_details[3]
+                                else:
+                                    # Remove unused bezier property
+                                    point.pop("handle_left", None)
 
-                                log.debug(
-                                    "updating interpolation mode point: co.X = %d to %d",
-                                    point["co"]["X"], interpolation)
+                                log.debug("updating interpolation mode point: co.X = %d to %d",
+                                          point["co"]["X"], interpolation)
                                 log.debug("use interpolation preset: %s", str(interpolation_details))
 
                         # Create new point (if needed)
@@ -456,41 +480,43 @@ class PropertiesModel(updates.UpdateInterface):
                             # Update or delete point
                             if new_value is not None:
                                 point["co"]["Y"] = float(new_value)
-                                log.debug(
-                                    "updating point: co.X = %d to value: %.3f",
-                                    point["co"]["X"], float(new_value))
+                                log.debug("updating point: co.X = %d to value: %.3f",
+                                          point["co"]["X"], float(new_value))
                             else:
                                 point_to_delete = point
                             break
 
-                        elif interpolation > -1 and point["co"]["X"] == previous_point_x:
-                            # Only update interpolation type (and the LEFT side of the curve)
+                        if interpolation > -1 and point["co"]["X"] == previous_point_x:
+                            # Only update the LEFT side of the curve (i.e. the previous point's right handle)
                             found_point = True
                             clip_updated = True
-                            point["interpolation"] = interpolation
-                            if interpolation == 0:
+                            if point.get("interpolation", 2) == 0 and interpolation_details:
                                 point["handle_right"] = point.get("handle_right") or {"Y": 0.0, "X": 0.0}
                                 point["handle_right"]["X"] = interpolation_details[0]
                                 point["handle_right"]["Y"] = interpolation_details[1]
 
-                            log.debug(
-                                "updating interpolation mode point: co.X = %d to %d",
-                                point["co"]["X"], interpolation)
-                            log.debug("use interpolation preset: %s", str(interpolation_details))
+                                log.debug("updating previous point (right handle): co.X = %d", point["co"]["X"])
+                                log.debug("use interpolation preset: %s", str(interpolation_details))
+                            else:
+                                # Remove unused bezier property
+                                point.pop("handle_right", None)
 
-                        elif interpolation > -1 and point["co"]["X"] == closest_point_x:
-                            # Only update interpolation type (and the RIGHT side of the curve)
+                        if interpolation > -1 and point["co"]["X"] == closest_point_x:
+                            # Only update the RIGHT side of the curve (i.e. the closest point's left handle
+                            # and interpolation mode (only the RIGHT SIDE POINT controls the interpolation mode)
                             found_point = True
                             clip_updated = True
                             point["interpolation"] = interpolation
-                            if interpolation == 0:
+                            if interpolation == 0 and interpolation_details:
                                 point["handle_left"] = point.get("handle_left") or {"Y": 0.0, "X": 0.0}
                                 point["handle_left"]["X"] = interpolation_details[2]
                                 point["handle_left"]["Y"] = interpolation_details[3]
+                            else:
+                                # Remove unused bezier property
+                                point.pop("handle_left", None)
 
-                            log.debug(
-                                "updating interpolation mode point: co.X = %d to %d",
-                                point["co"]["X"], interpolation)
+                            log.debug("updating interpolation mode point: co.X = %d to %d",
+                                      point["co"]["X"], interpolation)
                             log.debug("use interpolation preset: %s", str(interpolation_details))
 
                     # Delete point (if needed)
@@ -556,6 +582,14 @@ class PropertiesModel(updates.UpdateInterface):
                     except Exception:
                         log.warn('Invalid Reader value passed to property: %s', value, exc_info=1)
 
+            # Determine if waveforms are impacted by this change
+            has_waveform = False
+            waveform_file_id = None
+            if property_key == "volume":
+                if clip_data.get("ui", {}).get("audio_data", []):
+                    waveform_file_id = c.data.get("file_id")
+                    has_waveform = True
+
             # Reduce # of clip properties we are saving (performance boost)
             clip_data = {property_key: clip_data.get(property_key)}
             if object_id:
@@ -564,8 +598,13 @@ class PropertiesModel(updates.UpdateInterface):
             # Save changes
             if clip_updated:
                 # Save
+                c.data = clip_data
                 c.save()
 
+                # Update waveforms (if needed)
+                if has_waveform:
+                    get_audio_data({waveform_file_id: [c.id]})
+
                 # Update the preview
                 get_app().window.refreshFrameSignal.emit()
 
