@@ -76,6 +76,9 @@ class Preferences(QDialog):
         # Track metrics
         track_metric_screen("preferences-screen")
 
+        # Disable video caching
+        openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = False
+
         # Load all user values
         self.params = {}
         for item in self.settings_data:
@@ -214,7 +217,7 @@ class Preferences(QDialog):
                     widget.setMinimum(float(param["min"]))
                     widget.setMaximum(float(param["max"]))
                     widget.setValue(float(param["value"]))
-                    widget.setSingleStep(1.0)
+                    widget.setSingleStep(param.get("step", 1.0))
                     widget.setToolTip(param["title"])
                     widget.valueChanged.connect(functools.partial(self.spinner_value_changed, param))
 
@@ -224,7 +227,7 @@ class Preferences(QDialog):
                     widget.setMinimum(int(param["min"]))
                     widget.setMaximum(int(param["max"]))
                     widget.setValue(int(param["value"]))
-                    widget.setSingleStep(1)
+                    widget.setSingleStep(param.get("step", 1))
                     widget.setToolTip(param["title"])
                     widget.valueChanged.connect(functools.partial(self.spinner_value_changed, param))
 
@@ -278,9 +281,8 @@ class Preferences(QDialog):
                         value_list.append({"name": "Default", "value": ""})
                         for audio_device in get_app().window.preview_thread.player.GetAudioDeviceNames():
                             value_list.append({
-                                "name": "%s: %s" % (
-                                    audio_device.type, audio_device.name),
-                                "value": audio_device.name,
+                                "name": "%s: %s" % (audio_device[1], audio_device[0]),
+                                "value": audio_device[0],
                                 })
 
                     # Overwrite value list (for language dropdown)
@@ -361,6 +363,10 @@ class Preferences(QDialog):
                         v = value_item["value"]
                         i = value_item.get("icon", None)
 
+                        # Translate dropdown item (if needed)
+                        if param.get("translate_values"):
+                            k = _(value_item["name"])
+
                         # Override icons for certain values
                         # TODO: Find a more elegant way to do this
                         icon = None
@@ -510,7 +516,9 @@ class Preferences(QDialog):
             openshot.Settings.Instance().DE_LIMIT_HEIGHT_MAX = int(str(value))
 
         # Apply cache settings (if needed)
-        if param["setting"] in ["cache-limit-mb", "cache-scale", "cache-quality"]:
+        if param["setting"] in ["cache-limit-mb", "cache-scale", "cache-quality",
+                                "cache-ahead-percent", "cache-preroll-min-frames",
+                                "cache-preroll-max-frames", "cache-max-frames"]:
             get_app().window.InitCacheSettings()
 
         # Check for restart
@@ -634,6 +642,9 @@ class Preferences(QDialog):
         self.reject()
 
     def reject(self):
+        # Enable video caching
+        openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = True
+
         # Prompt user to restart openshot (if needed)
         if self.requires_restart:
             msg = QMessageBox()
