@@ -26,13 +26,14 @@
  """
 
 import json
+import time
 
 from PyQt5.QtCore import (
     Qt, QCoreApplication, QMutex, QTimer,
     QPoint, QPointF, QSize, QSizeF, QRect, QRectF,
 )
 from PyQt5.QtGui import (
-    QTransform, QPainter, QPixmap, QColor, QPen, QBrush, QCursor, QImage, QRegion
+    QTransform, QPainter, QIcon, QColor, QPen, QBrush, QCursor, QImage, QRegion
 )
 from PyQt5.QtWidgets import QSizePolicy, QWidget, QPushButton
 
@@ -51,14 +52,11 @@ class VideoWidget(QWidget, updates.UpdateInterface):
     # This method is invoked by the UpdateManager each time a change happens (i.e UpdateInterface)
     def changed(self, action):
         # Handle change
-        display_ratio_changed = False
-        pixel_ratio_changed = False
         if (action.key and action.key[0] in [
                 "display_ratio", "pixel_ratio"
                 ] or action.type in ["load"]):
             # Update display ratio (if found)
             if action.type == "load" and action.values.get("display_ratio"):
-                display_ratio_changed = True
                 self.aspect_ratio = openshot.Fraction(
                     action.values.get("display_ratio", {}).get("num", 16),
                     action.values.get("display_ratio", {}).get("den", 9))
@@ -66,7 +64,6 @@ class VideoWidget(QWidget, updates.UpdateInterface):
                     "Load: Set video widget display aspect ratio to: %s",
                     self.aspect_ratio.ToFloat())
             elif action.key and action.key[0] == "display_ratio":
-                display_ratio_changed = True
                 self.aspect_ratio = openshot.Fraction(
                     action.values.get("num", 16),
                     action.values.get("den", 9))
@@ -76,7 +73,6 @@ class VideoWidget(QWidget, updates.UpdateInterface):
 
             # Update pixel ratio (if found)
             if action.type == "load" and action.values.get("pixel_ratio"):
-                pixel_ratio_changed = True
                 self.pixel_ratio = openshot.Fraction(
                     action.values.get("pixel_ratio").get("num", 1),
                     action.values.get("pixel_ratio").get("den", 1))
@@ -84,7 +80,6 @@ class VideoWidget(QWidget, updates.UpdateInterface):
                     "Set video widget pixel aspect ratio to: %s",
                     self.pixel_ratio.ToFloat())
             elif action.key and action.key[0] == "pixel_ratio":
-                pixel_ratio_changed = True
                 self.pixel_ratio = openshot.Fraction(
                     action.values.get("num", 1),
                     action.values.get("den", 1))
@@ -273,11 +268,59 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         # Remove transform
         painter.resetTransform()
 
+    def update_title(self):
+        """Update the widget title"""
+        # Translate object
+        _ = get_app()._tr
+        rect = self.centeredViewport(self.width(), self.height())
+        scale = self.devicePixelRatioF()
+
+        # Display the playback speed in widget title
+        speed = 0.0
+        mode = self.win.preview_thread.player.Mode()
+        if mode != openshot.PLAYBACK_PAUSED:
+            speed = self.win.preview_thread.player.Speed()
+
+        # Find parent dockWidget (if any)
+        dock = None
+        if self.parent() and self.parent().parent():
+            # TODO: Find a better way to find the QDockWidget parent (if any)
+            dock = self.parent().parent()
+        else:
+            # Not a dock widget, ignore title
+            return
+
+        if self.settings.get("preview-fps"):
+            # Update window title with FPS output
+            dock.setWindowTitle(_("Video Preview") + _(" (Speed: %(speed)sx, Paint: %(pfps)s FPS, "
+                                                        "Render: %(rfps)s FPS, %(width)sx%(height)s)")
+                                % {"speed": speed,
+                                   "pfps": self.paint_fps,
+                                   "rfps": self.present_fps,
+                                   "width": rect.width() * scale,
+                                   "height": rect.height() * scale})
+        else:
+            # Restore window title
+            if not speed in [1, 0, -1]:
+                dock.setWindowTitle(_("Video Preview") + f" ({speed}x)")
+            else:
+                dock.setWindowTitle(_("Video Preview"))
+
     def paintEvent(self, event, *args):
         """ Custom paint event """
         event.accept()
         self.mutex.lock()
 
+        # Calculate "paint" FPS (and update widget title)
+        current_sec = time.localtime(time.time()).tm_sec
+        if current_sec != self.paint_fps_sec:
+            self.paint_fps = self.paint_fps_counter
+            self.update_title()
+            self.paint_fps_sec = current_sec
+            self.paint_fps_counter = 1
+        else:
+            self.paint_fps_counter += 1
+
         # Paint custom frame image on QWidget
         painter = QPainter(self)
         painter.setRenderHints(
@@ -299,8 +342,9 @@ class VideoWidget(QWidget, updates.UpdateInterface):
             pixSize.scale(event.rect().size(), Qt.KeepAspectRatio)
             self.curr_frame_size = pixSize
 
-            # Scale image
-            scaledPix = self.current_image.scaled(pixSize, Qt.KeepAspectRatio, Qt.SmoothTransformation)
+            # Scale image (take into account display scaling for High DPI monitors)
+            scale = self.devicePixelRatioF()
+            scaledPix = self.current_image.scaled(pixSize * scale, Qt.KeepAspectRatio, Qt.SmoothTransformation)
 
             # Calculate center of QWidget and Draw image
             painter.drawImage(viewport_rect, scaledPix)
@@ -518,15 +562,26 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         window_rect = QRectF(QPointF(0, 0), window_size)
 
         aspectRatio = self.aspect_ratio.ToFloat() * self.pixel_ratio.ToFloat()
-        viewport_size = QSizeF(aspectRatio, 1).scaled(window_size, Qt.KeepAspectRatio)
+        viewport_size = QSizeF(aspectRatio, 1).scaled(
+                            window_size, Qt.KeepAspectRatio
+                        ) * self.zoom
         viewport_rect = QRectF(QPointF(0, 0), viewport_size)
         viewport_rect.moveCenter(window_rect.center())
-
-        return viewport_rect.toRect()
+        # Always round up to next whole integer value
+        return viewport_rect.toAlignedRect()
 
     def present(self, image, *args):
         """ Present the current frame """
 
+        # Calculate "render" / "present" FPS
+        current_sec = time.localtime(time.time()).tm_sec
+        if current_sec != self.present_fps_sec:
+            self.present_fps = self.present_fps_counter
+            self.present_fps_sec = current_sec
+            self.present_fps_counter = 1
+        else:
+            self.present_fps_counter += 1
+
         # Get frame's QImage from libopenshot
         self.current_image = image
 
@@ -548,6 +603,10 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         # Ignore undo/redo history temporarily (to avoid a huge pile of undo/redo history)
         get_app().updates.ignore_history = True
 
+        # Disable video caching during drag operation (for performance reasons)
+        openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = False
+        log.debug('mousePressEvent: Stop caching frames on timeline')
+
     def mouseReleaseEvent(self, event):
         event.accept()
         """Capture mouse release event on video preview window"""
@@ -601,6 +660,10 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         # Inform UpdateManager to accept updates, and only store our final update
         get_app().updates.ignore_history = False
 
+        # Enable video caching again
+        openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = True
+        log.debug('mouseReleaseEvent: Start caching frames on timeline')
+
         # Add final update to undo/redo history
         if self.original_clip_data:
             get_app().updates.apply_last_action_to_history(self.original_clip_data)
@@ -1224,6 +1287,7 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         if self.transforming_clip and not clip_id:
             # Clear transform
             self.transforming_clip = None
+            self.transforming_clip_object = None
             need_refresh = True
 
         # Get new clip for transform
@@ -1232,12 +1296,12 @@ class VideoWidget(QWidget, updates.UpdateInterface):
             self.transforming_clip_object = win.timeline_sync.timeline.GetClip(clip_id)
             if self.transforming_clip and self.transforming_clip_object:
                 self.transforming_effect = None
+                self.transforming_effect_object = None
                 need_refresh = True
 
-        # Update the preview and reselct current frame in properties
+        # Update the preview and reselect current frame in properties
         if need_refresh:
             win.refreshFrameSignal.emit()
-            win.propertyTableView.select_frame(win.preview_thread.player.Position())
 
     def keyFrameTransformTriggered(self, effect_id, clip_id):
         """Handle the key frame transform signal when it's emitted"""
@@ -1249,7 +1313,9 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         if self.transforming_effect and not effect_id:
             # Clear transform
             self.transforming_effect = None
+            self.transforming_effect_object = None
             self.transforming_clip = None
+            self.transforming_clip_object = None
             need_refresh = True
 
         # Get new clip for transform
@@ -1271,7 +1337,7 @@ class VideoWidget(QWidget, updates.UpdateInterface):
     def regionTriggered(self, clip_id):
         """Handle the 'select region' signal when it's emitted"""
         # Clear transform
-        self.region_enabled = bool(not clip_id)
+        self.region_enabled = bool(clip_id)
         get_app().window.refreshFrameSignal.emit()
 
     def resizeEvent(self, event):
@@ -1281,7 +1347,7 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         self.delayed_resize_timer.start()
 
         # Pause playback (to prevent crash since we are fixing to change the timeline's max size)
-        self.win.actionPlay_trigger(force="pause")
+        self.win.PauseSignal.emit()
 
     def delayed_resize_callback(self):
         """Callback for resize event timer (to delay the resize event, and prevent lots of similar resize events)"""
@@ -1340,6 +1406,9 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         # Translate object
         _ = get_app()._tr
 
+        # Settings object
+        self.settings = get_app().get_settings()
+
         # Init aspect ratio settings (default values)
         self.aspect_ratio = openshot.Fraction(16, 9)
         self.pixel_ratio = openshot.Fraction(1, 1)
@@ -1379,18 +1448,27 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         self.resize_button.clicked.connect(self.resize_button_clicked)
         self.resize_button.setMouseTracking(True)
 
-        # Initialize cursors
-        self.cursors = {
-            "move": QPixmap(":/cursors/cursor_move.png"),
-            "resize_x": QPixmap(":/cursors/cursor_resize_x.png"),
-            "resize_y": QPixmap(":/cursors/cursor_resize_y.png"),
-            "resize_bdiag": QPixmap(":/cursors/cursor_resize_bdiag.png"),
-            "resize_fdiag": QPixmap(":/cursors/cursor_resize_fdiag.png"),
-            "rotate": QPixmap(":/cursors/cursor_rotate.png"),
-            "shear_x": QPixmap(":/cursors/cursor_shear_x.png"),
-            "shear_y": QPixmap(":/cursors/cursor_shear_y.png"),
-            "hand": QPixmap(":/cursors/cursor_hand.png"),
-            }
+        # FPS calculations
+        self.paint_fps = 0.0
+        self.paint_fps_counter = 1
+        self.paint_fps_sec = None
+        self.present_fps = 0.0
+        self.present_fps_counter = 1
+        self.present_fps_sec = None
+
+        # Load icon (using display DPI)
+        self.cursors = {}
+        for cursor_name in ["move",
+                            "resize_x",
+                            "resize_y",
+                            "resize_bdiag",
+                            "resize_fdiag",
+                            "rotate",
+                            "shear_x",
+                            "shear_y",
+                            "hand"]:
+            icon = QIcon(":/cursors/cursor_%s.png" % cursor_name)
+            self.cursors[cursor_name] = icon.pixmap(32, 32)
 
         # Mutex lock
         self.mutex = QMutex()
@@ -1411,6 +1489,13 @@ class VideoWidget(QWidget, updates.UpdateInterface):
         # Get a reference to the window object
         self.win = get_app().window
 
+        # Update title whenever playback speed changes.
+        self.win.PlaySignal.connect(self.update_title, Qt.QueuedConnection)
+        self.win.PlaySignal.connect(self.update_title, Qt.QueuedConnection)
+        self.win.PauseSignal.connect(self.update_title, Qt.QueuedConnection)
+        self.win.SpeedSignal.connect(self.update_title, Qt.QueuedConnection)
+        self.win.StopSignal.connect(self.update_title, Qt.QueuedConnection)
+
         # Show Property timer
         # Timer to use a delay before sending MaxSizeChanged signals (so we don't spam libopenshot)
         self.delayed_resize_timer = QTimer(self)
